%% Solve matching by MCMC sampling
% Data Driven Markov Chain Monte Carlo Sampling for matching
% by Jungmin Lee
function X = DDMCMC_icpr( M, E12 )

[list(:,1) list(:,2)] = find(E12);

% Assume exact graph matching
% Assume fully connected graph
% Data Driven : Use spectral information

%% parameters
startTemperature = 10;
finishTemperature = 0.1;
nSamplePerTemp = 15;
TempDecreasingFactor = 0.99;

%% Get the first eigenVector from affinity matrix
options.disp = 0;
% M = sparse(M);
[eigenVectorMat eigenValue] = eigs(M, 1, 'lm', options); % eigensolver for sparse matrix
eigenVector = abs(eigenVectorMat(:,1)); % take first eigenvector

%% initial state (suppose numA = numB)
numMatch = length(unique(list(:,1)));
reverseState = zeros(numMatch,1); % Memory Allocation
reverseTemp = zeros(numMatch,1);
proposal = zeros(length(list), 6);

proposal(:,1:2) = list;
proposal(:,3) = eigenVector;

currentState = randperm(numMatch)'; % initial state (random)
reverseState(currentState) = 1:numMatch; % Backward mapping
currentIdx = (currentState-1)*numMatch + (1:numMatch)';

x = zeros(length(list),1);
x(currentIdx) = 1;
currentEnergy = x'*M*x; % get energy finish here

bestState = currentState; % save best
bestEnergy = currentEnergy;

temperature = startTemperature;
count = 0;
bAccept = 1;

%% Sampling Loop
while temperature > finishTemperature 
    % Calculate proposal using current state
    if bAccept
        currentIdx = (currentState-1)*numMatch + (1:numMatch)';
        proposal(:,4) = eigenVector(proposal(:,1) + (currentState(proposal(:,1))-1)*numMatch);
        proposal(:,5) = eigenVector(reverseState(proposal(:,2)) + (proposal(:,2)-1)*numMatch);
        proposal(:,6) = eigenVector(reverseState(proposal(:,2)) + (currentState(proposal(:,1))-1)*numMatch);
        pdfEig = proposal(:,3) - proposal(:,4) - proposal(:,5) + proposal(:,6);
        pdfEig = pdfEig - min(pdfEig);
        pdfEig(currentIdx) = 0;

        % Make cdf & sampling
        pdfEig = pdfEig/sum(pdfEig);
        cdfEig = cumsum(pdfEig);   
    end
    r = rand;
    for i = 1:length(list)
        if r < cdfEig(i)
            probGo = pdfEig(i);
            break
        end
    end
    
    % Get index which will be changed
    proposeChange1 = i;
    proposeChange2 = reverseState(proposal(i,2)) + (currentState(proposal(i,1))-1)*numMatch;
    currentChange1 = proposal(i,1) + (currentState(proposal(i,1))-1)*numMatch;
    currentChange2 = reverseState(proposal(i,2)) + (proposal(i,2)-1)*numMatch;
       
    % Generate state proposal
    proposeState = currentState;
    proposeState(proposal(i,1)) = proposal(i,2);
    proposeState(reverseState(proposal(i,2))) = currentState(proposal(i,1));
    reverseTemp(proposeState) = 1:numMatch;
    proposeIdx = (proposeState-1)*numMatch + (1:numMatch)';
    
%     [ (1:length(proposal))' proposal(:,1:2) pdfEig ]
%     
%     [ currentState currentIdx ]
%     [ proposeState proposeIdx ]
%         
%     [ proposeChange1 proposeChange2 ]
%     [ currentChange1 currentChange2 ]
    
    % Calculate backward probability
    proposal(:,4) = eigenVector(proposal(:,1) + (proposeState(proposal(:,1))-1)*numMatch);
    proposal(:,5) = eigenVector(reverseTemp(proposal(:,2)) + (proposal(:,2)-1)*numMatch);
    proposal(:,6) = eigenVector(reverseTemp(proposal(:,2)) + (proposeState(proposal(:,1))-1)*numMatch);
    pdfEig = proposal(:,3) - proposal(:,4) - proposal(:,5) + proposal(:,6);
    pdfEig = pdfEig - min(pdfEig);
    pdfEig(proposeIdx) = 0;
    pdfEig = pdfEig/sum(pdfEig);
    probBack = pdfEig(currentChange1);

    % Calculate energy difference
    deltaEnergy = sum(M(proposeChange1, proposeIdx) + M(proposeChange2, proposeIdx)...
                 -M(currentChange1, currentIdx) - M(currentChange2, currentIdx))...
                 -M(proposeChange1,proposeChange2) + M(currentChange1,currentChange2);        
    proposeEnergy = currentEnergy + 2 * deltaEnergy; % get energy finish here      
    acceptRatio = exp(2*deltaEnergy/temperature)*probBack/probGo;
    count = count + 1;
        
%     x = zeros(length(list),1);
%     x(currentIdx) = 1;
%     x'*M*x
%     
%     currentEnergy
    
%     deltaEnergy
    
%     proposeEnergy
    
%     x = zeros(length(list),1);
%     x(proposeIdx) = 1;
%     x'*M*x

    % Apply MH rule
    if rand < min(acceptRatio, 1)
        bAccept = 1;
        currentState = proposeState;
        currentEnergy = proposeEnergy;
        reverseState = reverseTemp;       
        if currentEnergy > bestEnergy
            bestState = currentState;
            bestEnergy = currentEnergy;
        end
    else
        bAccept = 0;
    end
    
    if count > nSamplePerTemp*sqrt(numMatch)
        temperature = temperature*TempDecreasingFactor;
        count = 0;
    end
    
%     pause
end

answer = (bestState-1)*numMatch + (1:numMatch)';

X = zeros(size(E12,1), size(E12,2));
X(answer) = 1;
X = X(:);

% bestEnergy