function [maxMember, maxSco] = tabu_yumin_reproduce(aff,member0,nNode)
% 2016/6/7 Yumin Suh
%
% INPUT
% aff: (nMatch x nMatch) affinity matrix
% member0: (nMatch x 1) initial match at [1,nNode]
% nNode: (scalar) number of nodes in a input graph
%
% OUTPUT
% maxMember:(nNode x 1) best discovered solution
% maxSco: (scalar)

% rng(1);

% Debug
% assert(all(all(aff==aff')));

% tabu parameters
param.nSearchMovein = 5;
param.nSearchMoveout = 5;
param.tabuMin = 2;
param.tabuDur = 4;  % !!
param.nNonImpIterMax = 1500;
param.penalty = -9;
param.iterMax = 10^5;

% constants
nMatch = nNode^2;
% nNode = nNode;

% variables
% alpha = zeros(nMatch, 1);
tabuIter = zeros(nMatch,1);
moveGains = nan(param.nSearchMoveout, param.nSearchMovein);

%% Initialize variable
curMember = member0(1:nNode);
curOutlier = member0(nNode+1:end);
curSco = sum(sum(aff(curMember,curMember)))/2;
maxMember = [];
maxSco = curSco;
alpha = sum(aff(:,curMember),2);
nonImpIter = 0;

%% Main iteration
for iter = 1 : param.iterMax
    if(nonImpIter > param.nNonImpIterMax), disp('normal termination'); break;  end
    
    % Sort alpha
    [sortedAlphaIn, sortedAlphaInIdx] = sort(alpha(curMember),'ascend');
    [sortedAlphaOut, sortedAlphaOutIdx] = sort(alpha(curOutlier),'descend');
    
    % Calculate move gains
    moveouts = curMember(sortedAlphaInIdx(1:param.nSearchMoveout));
    moveins = curOutlier(sortedAlphaOutIdx(1:param.nSearchMovein));
    for iMoveout = 1 : param.nSearchMoveout
        for iMovein = 1 : param.nSearchMovein
            moveout = moveouts(iMoveout);
            movein = moveins(iMovein);
            moveGains(iMoveout,iMovein) = ...
                sortedAlphaOut(iMovein) - sortedAlphaIn(iMoveout) - aff(movein, moveout);
        end
    end
    
    % Update states
    % select movein and moveout
    [iMoveout, iMovein, movegain] = ...
        selectMoveInOut(moveGains, tabuIter(moveouts)>iter, tabuIter(moveins)>iter, curSco, maxSco);
    if(isnan(iMoveout) || isnan(iMovein) || isnan(movegain)),  break;  end
    theMoveout = moveouts(iMoveout);
    theMovein = moveins(iMovein);
    % update alpha
    alpha = updateAlpha(alpha, theMoveout, theMovein, aff);
    % update tabuIter
    tabuIter = updateTabuIter(iter, tabuIter, theMoveout, theMovein, param);
    % update curX and curSco
    [curMember, curOutlier, curSco] = updateX(curMember, curOutlier, curSco, theMoveout, theMovein, movegain);
    % update maxX and maxSco
    if(curSco > maxSco),    maxMember = curMember;    maxSco = curSco;    nonImpIter = 0;
    else    nonImpIter = nonImpIter + 1;
    end
    % Debug
%     assert(all(alpha-sum(aff(:,curX),2)<1e-6));
%     assert(sum(sum(aff(curX,curX)))/2-curSco<1e-6);
end

end



function alpha = updateAlpha(alpha, moveout, movein, aff)
% assert(all(all(aff==aff')));
% assert(all(aff(1:length(aff)+1:end)==0));
    tmpAlpha = alpha + aff(:,movein) - aff(:,moveout);
    tmpAlpha(movein) = alpha(movein) - aff(movein,moveout);
    tmpAlpha(moveout) = alpha(moveout) + aff(movein,moveout);
    alpha = tmpAlpha;
end

function tabuIter = updateTabuIter(iter, tabuIter, movein, moveout, param)
    tabuIter(movein) = iter + ceil(param.tabuMin + param.tabuDur*rand);
    tabuIter(moveout) = iter + ceil(param.tabuMin + param.tabuDur*rand);
end

function [curMember, curOutlier, curSco] = updateX( curMember, curOutlier, curSco, moveout, movein, movegain)
    curSco = curSco + movegain;
    curMember(curMember==moveout) = movein;
    curOutlier(curOutlier==movein) = moveout;
end